% jps regressions for bauer hamilton discussion

clear all 
close all

% a little picture of simulated return 

rng(3)
r = 10-(1:30)'/3 + 4*randn(30,1);
[bv,sebv,R2v,R2vadj,v,F] = olsgmm(r,[ones(30,1) (1:30)'],0,0);
disp(bv)
disp(bv./sebv)

figure
plot((1:30)',r,'linewidth',2);
xlabel('time')
ylabel('return')
print -depsc simulated_trend.eps


% regressions 
% get data

x = load('jps_updated_cleaned_for_matlab.csv'); 
yield05 = x(:,1); 
yield1 = x(:,2);
yield2 = x(:,3);
yield3 = x(:,4);
yield4 = x(:,5);
yield5 = x(:,6);
yield6 = x(:,7);
yield7 = x(:,8);
yield8 = x(:,9);
yield9 = x(:,10);
yield10 = x(:,11);
p05 = x(:,12);
p1 = x(:,13);
p2 = x(:,14);
p3 = x(:,15);
p4 = x(:,16);
p5 = x(:,17);
p6 = x(:,18);
p7 = x(:,19);
p8 = x(:,20);
p9 = x(:,21);
p10 = x(:,22);
GRO_UPDATED = x(:,23);
INF = x(:,24);
GRO = x(:,25);
yyyymm = x(:,26);
year = x(:,27);
xr2 = x(:,28);
xr3 = x(:,29);
xr4 = x(:,30);
xr5 = x(:,31);
xr6 = x(:,32);
xr7 = x(:,33);
xr8 = x(:,34);
xr9 = x(:,35);
xr10 = x(:,36);
xr.avg = x(:,37);
PC1 = x(:,38);
PC2 = x(:,39);
PC3 = x(:,40);
PC4 = x(:,41);
PC5 = x(:,42);

js_year = floor(yyyymm/100);
js_mo = yyyymm-100*js_year;
js_date = datenum(js_year,js_mo,1); 

% transformations

T = size(p1,1);
p = log([p1 p2 p3 p4 p5 p6 p7 p8 p9 p10]/100); 
y = -p./(ones(T,1)*[1 2 3 4 5 6 7 8 9 10]);
r = p(13:end,1:end-1)-p(1:end-12,2:end); %r rx are shorter
rx = r - y(1:end-12,1)*ones(1,9); % rx is only 9 wide
f = [-p(:,1) p(:,1:end-1)-p(:,2:end)];
fs = f(:,2:end)-y(:,1)*ones(1,9); % fs is only 9 wide 

[Q, L] = eig(cov(fs));
% Q L Q' = cov(fs)

pc = fs*Q; % Q'f_t = factors_t,

% plot data

figure
plot(js_date,fs)
datetick('x')
title('JPS Forward-spot spreads'); 
legend('2','3','4','5','6','7','8','9','10')
print -depsc2 jps_fs.eps


figure
plot(js_date,[PC1 PC2 PC3],'linewidth',2)
datetick('x')
title('JPS Principal Components'); 
legend('1','2','3')
print -depsc2 jps_pc.eps

figure
plot(js_date,100*y)
datetick('x')
title('JPS yields'); 
legend('1','2','3','4','5','6','7','8','9','10')
print -depsc2 jps_y.eps

figure; 
plot(js_date,[INF GRO],'linewidth',2); 
hold on
plot(js_date,0*js_date); 
axis([-inf inf -4 6]);
datetick('x'); 
legend('Inflation','Growth'); 
print -depsc2 macro_data.eps

% look at data
if 0;
    mos = {'Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec'};
    
    for mosel = 1:12;
        fs_sel = fs(js_mo==mosel,:);
        y_sel = y(js_mo==mosel,:);
        js_date_sel = js_date(js_mo==mosel,:);
        js_year_sel = js_year(js_mo==mosel);
        
        figure
        plot((2:10)', 100*fs_sel','-v','linewidth',2);
        title(['Forward spot spreads in ' char(mos(mosel))])
        text(10*ones(size(fs_sel,1),1),100*fs_sel(:,end),num2str(js_year_sel,'%4.0f'))
        xlabel('Maturity')
    end
end;

% search for data horrors

bad_index = sum(((fs(:,3:end)-fs(:,2:end-1))-(fs(:,2:end-1)-fs(:,1:end-2))).^2,2);
[B,I] = sort(bad_index); 
I = I(end:-1:1); 
figure;
for indx = 1:6;
    subplot(2,3,indx);
    plot((2:10)', 100*fs(I(indx),:)', '-v','linewidth',2)
    hold on;
    plot([5 5]',[-2 6]','--k')
    plot([2 10]',[0 0]','-k')
    
    title([ num2str(js_mo(I(indx)),'%2.0f') ' / ' num2str(js_year(I(indx)),'%4.0f')] )
    xlabel('Maturity');
    axis([2 10 -2 12]) 
end;
print -depsc2 jps_horror.eps;

bad_index = sum(((y(:,3:end)-y(:,2:end-1))-(y(:,2:end-1)-y(:,1:end-2))).^2,2);
[B,I] = sort(bad_index); 
I = I(end:-1:1); 
figure;
for indx = 1:6;
    subplot(2,3,indx);
    plot((2:10)', 100*(y(I(indx),2:end)-y(I(indx),1)*ones(1,9))', '-v','linewidth',2)
    hold on;
    plot([5 5]',[-2 2]','--k')
    plot([2 10]',[0 0]','-k')
    
    title([ num2str(js_mo(I(indx)),'%2.0f') ' / ' num2str(js_year(I(indx)),'%4.0f')] )
    xlabel('Maturity');
    axis([2 10 0 inf]) 
end;

% jps regressions

disp('JPS regressions')

disp('\begin{tabular}{lr|ccc|cc|c|l}');
disp('Method & & PC1 & PC2 & PC3 & GRO &  INF & trend & $R^2$ \\');
disp('\hline'); 
% all returns on JS 3 PC

xlbls={'PC1','PC2','PC3'};
ylbls={'Erx'};
rhv = [PC1 PC2 PC3];
rhv = [ones(size(rhv,1),1) rhv]; 
[bj,sebj,R2j,bav,sebav,R2av,R2sameb,bov,sebov,R2ov] = f_doregs(100*mean(rx,2),rhv(1:end-12,:),xlbls,ylbls,0);

fit_pc = [ rhv]*bov; 

fprintf('%10.10s',  'Jan-Jan'); 
fprintf(' & %10.10s', ' b' ); 
fprintf(' &  %4.2f  ', bj(2:end)); 
fprintf(' &  %4.4s  ', ' ', ' ');
fprintf(' &  %4.4s  ', ' ');
fprintf(' &  %4.2f  ', R2j); 
fprintf(' \\\\ \n'); 
fprintf('  %10.10s', ' '); 
fprintf(' &%10.10s', ' t' ); 
fprintf(' & (%4.2f) ', bj(2:end)./sebj(2:end)); 
fprintf(' &  %4.4s  ', ' ', ' ',' ');
fprintf(' &  %4.4s  ', ' ');
fprintf(' \\\\ \n'); 

fprintf('%10.10s',  'Av Mo.' );
fprintf(' & %10.10s', ' b' ); 
fprintf(' &   %4.2f  ', bav(2:end)); 
fprintf(' &   %4.4s  ', ' ',' ');
fprintf(' &  %4.4s  ', ' ');
fprintf(' &   %4.2f  (av. mo.)', R2av); 
fprintf(' \\\\ \n'); 
fprintf('%10.10s', ' ' ); 
fprintf(' & %10.10s', ' t' );
fprintf(' & (%4.2f) ', bj(2:end)./sebj(2:end)); 
fprintf(' &  %4.12s ', ' ',' ');
fprintf(' &  %4.4s  ', ' ');
fprintf(' &  %4.2f (same b)', R2sameb); 
fprintf(' \\\\ \n'); 

fprintf('%10.10s',  'Overlap' );

fprintf(' & %10.10s', ' b' ); 
fprintf(' &  %4.2f  ', bov(2:end)); 
fprintf(' &  %4.2s  ', ' ',' ');
fprintf(' &  %4.4s  ', ' ');
fprintf(' &  %4.2f  ', R2ov); 
fprintf(' \\\\ \n'); 
fprintf('%10.10s', ' ' ); 
fprintf(' & %10.10s', ' t' );
fprintf(' & (%4.2f) ', bov(2:end)./sebov(2:end)); 
fprintf(' &  %4.2s  ', ' ',' ');
fprintf(' &  %4.4s  ', ' ');
fprintf(' &  %4.2s  ', ' '); 
fprintf(' \\\\ \n'); 
disp('\hline')

% gro and inf

rhv = [GRO INF];
rhv = [ones(size(rhv,1),1) rhv]; 
xlbls={'GRO','INF'};
ylbls={'Erx'};
[bj,sebj,R2j,bav,sebav,R2av,R2sameb,bov,sebov,R2ov] = f_doregs(100*mean(rx,2),rhv(1:end-12,:),xlbls,ylbls,0);

fit_macro = [ rhv]*bov;

fprintf('%10.10s',  'Jan-Jan'); 
fprintf(' & %10.10s', ' b' ); 
fprintf(' & %4.2s ', ' ', ' ',' ');
fprintf(' &  %4.2f  ', bj(2:end));
fprintf(' &  %4.4s  ', ' ');
fprintf(' &  %4.2f  ', R2j); 
fprintf(' \\\\ \n'); 
fprintf('%10.10s', ' '); 
fprintf(' & %10.10s', ' t' ); 
fprintf(' &  %4.2s  ', ' ', ' ',' ');
fprintf(' & (%4.2f) ', bj(2:end)./sebj(2:end)); 
fprintf(' &  %4.4s  ', ' ');
fprintf(' & %10.10s ', ' ');
fprintf(' \\\\ \n'); 

fprintf('%10.10s',  'Av Mo.' );
fprintf(' & %10.10s', ' b' ); 
fprintf(' &  %4.2s ', ' ',' ',' ');
fprintf(' &  %4.2f  ', bav(2:end)); 
fprintf(' &  %4.4s  ', ' ');
fprintf(' &  %4.2f (av. mo.)', R2av); 
fprintf(' \\\\ \n'); 
fprintf('%10.10s', ' ' ); 
fprintf(' & %10.10s', ' t' );
fprintf(' &  %4.2s  ', ' ',' ', ' ');
fprintf(' & (%4.2f) ', bj(2:end)./sebj(2:end)); 
fprintf(' &  %4.4s  ', ' ');

fprintf(' & %4.2f (same b)', R2sameb); 
fprintf(' \\\\ \n'); 

fprintf('%10.10s',  'Overlap' );

fprintf(' & %10.10s', ' b' ); 
fprintf(' &  %4.2s  ', ' ',' ', ' ');
fprintf(' &  %4.2f  ', bov(2:end)); 
fprintf(' &  %4.4s  ', ' ');
fprintf(' &  %4.2f  ', R2ov); 
fprintf(' \\\\ \n'); 
fprintf('%10.10s', ' ' ); 
fprintf(' & %10.10s', ' t' );
fprintf(' &  %4.2s  ', ' ',' ', ' ');
fprintf(' & (%4.2f) ', bov(2:end)./sebov(2:end));
fprintf(' &  %4.4s  ', ' ');
fprintf(' & %10.2s ', ' '); 
fprintf(' \\\\ \n'); 
disp('\hline')


% JS 3 PC, GRO INF

xlbls={'PC1','PC2','PC3','GRO','INF'};
ylbls={'Erx'};
rhv = [PC1 PC2 PC3 GRO INF];
rhv = [ones(size(rhv,1),1) rhv]; 
[bj,sebj,R2j,bav,sebav,R2av,R2sameb,bov,sebov,R2ov] = f_doregs(100*mean(rx,2),rhv(1:end-12,:),xlbls,ylbls,0);

fit_both = [ rhv]*bov; 

fprintf('%10.10s',  'Jan-Jan'); 
fprintf(' & %10.10s', ' b' ); 
fprintf(' & %4.2f ', bj(2:end)); 
fprintf(' &  %4.2s  ', ' ');
fprintf(' & %4.2f ', R2j); 
fprintf(' \\\\ \n'); 
fprintf('%10.10s', ' '); 
fprintf(' & %10.10s', ' t' ); 
fprintf(' & (%4.2f) ', bj(2:end)./sebj(2:end));
fprintf(' &  %4.4s  ', ' ');
fprintf(' &  %4.2s  ', ' ');
fprintf(' \\\\ \n'); 

fprintf('%10.10s',  'Av Mo.' );
fprintf(' & %10.10s', ' b' ); 
fprintf(' &  %4.2f  ', bav(2:end));
fprintf(' &  %4.4s  ', ' ');
fprintf(' &  %4.2f (av. mo.)', R2av); 
fprintf(' \\\\ \n'); 
fprintf('%10.10s', ' ' ); 
fprintf(' & %10.10s', ' t' );
fprintf(' & (%4.2f) ', bj(2:end)./sebj(2:end)); 
fprintf(' &  %4.4s  ', ' ');
fprintf(' &  %4.2f  (same b)', R2sameb); 
fprintf(' \\\\ \n'); 

fprintf('%10.10s',  'Overlap' );

fprintf(' & %10.10s', ' b' ); 
fprintf(' &  %4.2f  ', bov(2:end));
fprintf(' &  %4.4s  ', ' ');
fprintf(' &  %4.2f  ', R2ov); 
fprintf(' \\\\ \n'); 
fprintf('%10.10s', ' ' ); 
fprintf(' & %10.10s', ' t' );
fprintf(' & (%4.2f) ', bov(2:end)./sebov(2:end)); 
fprintf(' &  %4.4s  ', ' ');
fprintf(' &  %4.2s  ', ' '); 
fprintf(' \\\\ \n'); 
disp('\hline')

% JS 3 PC, GRO INF, trend

xlbls={'PC1','PC2','PC3','GRO','INF','trend'};
ylbls={'Erx'};
rhv = [PC1 PC2 PC3 GRO INF (1:size(INF,1))'];
rhv = [ones(size(rhv,1),1) rhv]; 
[bj,sebj,R2j,bav,sebav,R2av,R2sameb,bov,sebov,R2ov] = f_doregs(100*mean(rx,2),rhv(1:end-12,:),xlbls,ylbls,0);

fit_trend = [ rhv]*bov; 

fprintf('%10.10s',  'Overlap' );

fprintf(' & %10.10s', ' b' ); 
fprintf(' &  %4.2f  ', bov(2:end)); 
fprintf(' &  %4.2f  ', R2ov); 
fprintf(' \\\\ \n'); 
fprintf('%10.10s', ' ' ); 
fprintf(' & %10.10s', ' t' );
fprintf(' & (%4.2f) ', bov(2:end)./sebov(2:end)); 
fprintf(' &  %4.2s  ', ' '); 
fprintf(' \\\\ \n'); 
disp('\hline')



% JS PC, trend

xlbls={'PC1','trend'};
ylbls={'Erx'};
rhv = [PC1 PC2 (1:size(INF,1))'];
rhv = [ones(size(rhv,1),1) rhv]; 
[bj,sebj,R2j,bav,sebav,R2av,R2sameb,bov,sebov,R2ov] = f_doregs(100*mean(rx,2),rhv(1:end-12,:),xlbls,ylbls,0);

fit_pc12_trend = [rhv]*bov; 

fprintf('%10.10s',  'Overlap' );

fprintf(' & %10.10s', ' b' ); 
fprintf(' &  %4.2f  ', bov(2:3));
fprintf(' &  %4.2s  ', ' '); 
fprintf(' &  %4.2s  ', ' '); 
fprintf(' &  %4.2s  ', ' ');
fprintf(' &  %4.2f  ', bov(end));
fprintf(' &  %4.2f  ', R2ov); 
fprintf(' \\\\ \n'); 
fprintf('%10.10s', ' ' ); 
fprintf(' & %10.10s', ' t' );
fprintf(' & (%4.2f) ', bov(2:3)./sebov(2:3));
fprintf(' &  %4.2s  ', ' '); 
fprintf(' &  %4.2s  ', ' '); 
fprintf(' &  %4.2s  ', ' '); 
fprintf(' & (%4.2f) ', bov(end)./sebov(end));
fprintf(' &  %4.2s  ', ' '); 
fprintf(' \\\\ \n'); 
disp('\hline')


figure; 
subplot(2,2,1)
plot(js_date(13:end),100*mean(rx,2), 'linewidth',2); 
hold on
plot(js_date(13:end),fit_pc(1:end-12),'-r', 'linewidth',2);
title('3 PC')
legend('rx_{t+1}','a+b''x_t')
datetick('x');

subplot(2,2,2)
plot(js_date(13:end),100*mean(rx,2), 'linewidth',2); 
hold on
plot(js_date(13:end),fit_macro(1:end-12),'-r', 'linewidth',2);
title('GRO and INF')
datetick('x');

subplot(2,2,3)
plot(js_date(13:end),100*mean(rx,2), 'linewidth',2); 
hold on
plot(js_date(13:end),fit_both(1:end-12),'-r', 'linewidth',2);
title('3 PC, GRO and INF')
datetick('x');

subplot(2,2,4)
plot(js_date(13:end),100*mean(rx,2), 'linewidth',2); 
hold on
plot(js_date(13:end),fit_pc12_trend(1:end-12),'-r', 'linewidth',2);
title('PC1, PC2, Trend')
datetick('x');

print -depsc2 JPS_plots.eps

xlbls={'PC1','PC2','PC3','GRO','INF'};
ylbls={'rx1','rx2','rx3','rx4','rx5','rx6','rx7','rx8','rx9','rx10'};
rhv = [PC1 PC2 PC3 GRO INF];
rhv = [ones(size(rhv,1),1) rhv]; 
[bj,sebj,R2j,bav,sebav,R2av,R2sameb,bov,sebov,R2ov] = f_doregs(100*rx,rhv(1:end-12,:),xlbls,ylbls,0);

fits = [ rhv]*bov; 
figure;
plot(js_date,fits(:,end:-1:1)); 
%legend('rx10','rx9','rx8','rx7','rx6','rx5','rx4','rx3','rx2','location','EastOutside');
axis([-inf inf -8 20])
datetick('x');
title('Expected returns from PC1 PC2 PC3 GRO INF') 
print -depsc2 fittedr.eps

[Q, L] = eig(cov(fits));
disp('eigenvalues of expected returns -- fractions of variance') 
disp( (abs(diag(L)))./sum((abs(diag(L))))*100); 
% is there a second one in this data? Macro / finance? 

disp('Q matrix -- columns are weights'); 
disp(Q); 

Erfactor = fits*(Q(end,:)');
figure 
plot(js_date,Erfactor); 
title ('Expected return factor') 
datetick('x')

xlbls={'Er'};
ylbls={'rx1','rx2','rx3','rx4','rx5','rx6','rx7','rx8','rx9','rx10'};
rhv = [Erfactor];
[bj,sebj,R2j,bav,sebav,R2av,R2sameb,bov,sebov,R2ov] = f_doregs(100*rx,rhv(1:end-12,:),xlbls,ylbls,0);
fits = [ rhv]*bov; 
disp('regressions of returns on er factor'); 
disp(bov); 
gammas = bov; % save for later


figure;
plot(js_date,fits(:,end:-1:1)); 
%legend('rx10','rx9','rx8','rx7','rx6','rx5','rx4','rx3','rx2','location','EastOutside');
axis([-inf inf -8 20])
datetick('x');
title('Expected returns from Er factor') 
print -depsc2 fittedr_factor.eps

% find factor shocks
xlbls={'PC','PC2','PC3','GRO','INF'};
ylbls={'PC','PC2','PC3','GRO','INF'};
lhv = [PC1 PC2 PC3 GRO INF];
rhv = [PC1 PC2 PC3 GRO INF];
rhv = [ones(size(rhv,1),1) rhv]; 

[bj,sebj,R2j,bav,sebav,R2av,R2sameb,bov,sebov,R2ov] = f_doregs(lhv(13:end,:),rhv(1:end-12,:),xlbls,ylbls,0);
factor_shocks = lhv(13:end,:) - rhv(1:end-12,:)*bov; 
covrf = (100*(rx-ones(size(rx,1),1)*mean(rx)))'*factor_shocks/size(rx,1); 

figure
plot(-covrf,gammas'*ones(1,5),'-v','linewidth',2); 
hold on
xlabel('Covariance of excess return with factors')
ylabel('Expected return factor loading \gamma'); 
legend('PC1','PC2','PC3','GRO','INF', 'location','southeast')
print -depsc2 ervscov.eps;

% run cross sectional regression 
lambdas = gammas'\(-covrf);
disp('cross sectional regression')
disp(lambdas)

stop % intentional error to stop script execution

%-----------------------------
% Program stops here. Below some investigations of CP style regressions
% but they failed once I saw the big forward rate measurement errors
% If you revive this section, you need to fix calls to doregs below 
% ---------------------------

disp(' ****** ') ;

% average of all returns

xlbls={'fs2','fs3','fs4','fs5','fs6','fs7','fs8','fs9','fs10'};
ylbls={'Erx'};

disp('average return across maturities')
[bj,sebj,R2j,bav,sebav,R2av,bov,sebov,R2ov] = f_doregs(mean(rx,2),fs(1:end-12,:),xlbls,ylbls);

figure; 
plot(bj(2:end),'linewidth',2); 
hold on
errorbar(bj(2:end),sebj(2:end));
plot((0:10),0*(0:10),'-k')
title('January')

figure; 
plot(bav(2:end),'linewidth',2); 
hold on
errorbar(bav(2:end),sebav(2:end));
plot((0:10),0*(0:10),'-k')
title('Average')

figure; 
plot(bov(2:end),'linewidth',2); 
hold on
errorbar(bov(2:end),sebov(2:end));
plot((0:10),0*(0:10),'-k')
title('Overlapping')



%all returns, all forwards

xlbls={'fs2','fs3','fs4','fs5','fs6','fs7','fs8','fs9','fs10'};
ylbls={'rx2','rx3','rx4','rx5','rx6','rx7','rx8','rx9','rx10'};
[bj,sebj,R2j,bav,sebav,R2av,bov,sebov,R2ov] = f_doregs(rx,fs(1:end-12,:),xlbls,ylbls);

figure; 
hold on
plot((2:5)',bj(2:5,1:4),'linewidth',2); 
plot((5:size(bj,1))',bj(5:end,1:4),'--','linewidth',2); 
plot((2:size(bj,1))',bj(2:end,4:end),'--','linewidth',2); 
legend('r2','r3','r4','r5','r6','r7','r8','r9','r10')

figure; 
hold on
plot((2:5)',bov(2:5,1:4),'linewidth',2); 
plot((5:size(bov,1))',bov(5:end,1:4),'--','linewidth',2); 
plot((2:size(bov,1))',bov(2:end,4:end),'--','linewidth',2); 
legend('r2','r3','r4','r5','r6','r7','r8','r9','r10')



stop 

% average returns on fs level and slope

xlbls={'fs'};
ylbls={'Erx'};

fslev =  mean(fs,2);
fs_slope = mean(fs(:,2:4),2)-mean(fs(:,5:end),2);
disp('average return on fs level')
[bj,sebj,R2j,bav,sebav,R2av,bov,sebov,R2ov] = f_doregs(mean(rx,2),[fslev(1:end-12,:)],xlbls,ylbls);

xlbls={'fs l-s'};
ylbls={'Erx'};
disp('average return on fs slope')
[bj,sebj,R2j,bav,sebav,R2av,bov,sebov,R2ov] = f_doregs(mean(rx,2),[fs_slope(1:end-12,:)],xlbls,ylbls);

xlbls={'fs','fs l-s'};
ylbls={'Erx'};
disp('average return on fs level and slope')
[bj,sebj,R2j,bav,sebav,R2av,bov,sebov,R2ov] = f_doregs(mean(rx,2),[fslev(1:end-12,:) fs_slope(1:end-12,:)],xlbls,ylbls);

